
%   University of Ljubljana
%   Faculty of Computer and Information Science
%   National Institute of Chemistry Ljubljana
%
%	Stohastic modelling and simulation of oscillator constructed in mammalian cell Hek293
%	
%	Copyright (C) 2010, iGEM team Slovenia

% Delay implemented only as a nonlinear function of mRNA delay time of transport
% and a delay of the activator and repressor time of protein binding

function stohastic_delay_02( )

	format long;
%%%%%%%%% System reactions %%%%%%%%%%
% Common type of sharing reactions (happening in all of three plasmids)

%% 1. plasmid %%
%  01:	 pT7_rnapt7  + RNAPT7	->  pT7_rnapt7*								( activation )
%  02:	 pT7_rnapt7*			->  pT7_rnapt7 + RNAPT7						( dissociation of RNAP ligand )
%  03:	 pT7_rnapt7  + NICTAL	->  pT7_rnapt7'								( repressor binding on promotor )
%  04:	 pT7_rnapt7'			->  pT7_rnapt7 + NICTAL						( dissociation of repressor protein )
%  05:	 pT7_rnapt7*			->  mRNA_nuc_rnapt7	+ pT7_rnapt7 + RNAPT7	( transcription	)
%  06:	 mRNA_cyt_rnapt7		->  RNAPT7 + mRNA_cyt_rnapt7				( translation )
%  07:	 mRNA_cyt_rnapt7		->  0										( mRNA degradation )
%  08:	 RNAPT7					->  0										( protein degradation )

%% 2. plasmid %%
%  09:	 pT7_nictal  + RNAPT7	->  pT7_nictal* 
%  10:	 pT7_nictal*			->  pT7_nictal + RNAPT7
%  11:	 pT7_nictal  + NICTAL	->  pT7_nictal'
%  12:	 pT7_nictal'			->  pT7_nictal + NICTAL
%  13:	 pT7_nictal*			->  mRNA_nuc_nictal	+ pT7_nictal + RNAPT7
%  14:	 mRNA_cyt_nictal		->  NICTAL + mRNA_cyt_nictal
%  15:	 mRNA_cyt_nictal		->  0
%  16:	 NICTAL					->  0

%% 3. plasmid %%
%  17:	 pT7_gfp  + RNAPT7		->  pT7_gfp* 
%  18:	 pT7_gfp*				->  pT7_gfp + RNAPT7
%  19:	 pT7_gfp  + NICTAL		->  pT7_gfp'
%  20:	 pT7_gfp'				->  pT7_gfp + NICTAL
%  21:	 pT7_gfp*				->  mRNA_nuc_gfp + pT7_gfp + RNAPT7
%  22:	 mRNA_cyt_gfp			->  GFP + mRNA_cyt_gfp
%  23:	 mRNA_cyt_gfp			->  0
%  24:	 GFP					->  0

%% Independent region %%
%  25:  pCMV					->  RNAPT7 + pCMV	( constant minimal production of RNAPT7 - simplify )

%% added reactions to complete delay for transport mRNA outside the nucleus
%  26:	mRNA_nuc_rnapt7			->	mRNA_cyt_rnapt7
%  27:  mRNA_nuc_nictal			->	mRNA_cyt_nictal
%  28:	mRNA_nuc_gfp			->	mRNA_cyt_gfp

%% degradation of bounded proteins
%  29:	pT7_rnapt7'				->	pT7_rnapt7
%  30:	pT7_nictal'				->	pT7_nictal
%  31:	pT7_gfp'				->	pT7_gfp

	% number of reactions
	R = 31;


%% vector of species
%
%		1	[	pT7_rnapt7		]
%  		2	[	pT7_rnapt7*		]
%  		3	[	pT7_rnapt7'		]
%		4	[	pT7_nictal		]
%  		5	[	pT7_nictal*		]
%  		6	[	pT7_nictal'		]
%		7	[	pT7_gfp			]
%  		8	[	pT7_gfp*		]
%  		9	[	pT7_gfp'		]
%		10	[	RNAPT7			]
%		11	[	NICTAL			]
%		12	[	GFP				]
%  		13	[	mRNA_cyt_rnapt7	]
%  		14	[	mRNA_cyt_nictal	]
%		15	[	mRNA_cyt_gfp	]
%		16	[	pCMV			]
%		17	[	mRNA_nuc_rnapt7	]
%		18	[	mRNA_nuc_nictal	]
%		19	[	mRNA_nuc_gfp	]
       
	% number of species
	S = 19;

	
%% tunned stoichiometric matrix
%  defined via reactions above (exc. in translation we could not used simply mass action law)
%  including cmv process ( constant production of RNAPT7 - reaction n.25)
	
	v = zeros(S,R);
%                1   2   3   4   5   6   7   8   9  10  11  12  13  14  15  16  17  18  19
	v(:,1)  = [ -1,  1,  0,  0,  0,  0,  0,  0,  0, -1,  0,  0,  0,  0,  0,  0,  0,  0,  0 ]';
	v(:,2)  = [  1, -1,  0,  0,  0,  0,  0,  0,  0,  1,  0,  0,  0,  0,  0,  0,  0,  0,  0 ]';
	v(:,3)  = [ -1,  0,  1,  0,  0,  0,  0,  0,  0,  0, -1,  0,  0,  0,  0,  0,  0,  0,  0 ]';
	v(:,4)  = [  1,  0, -1,  0,  0,  0,  0,  0,  0,  0,  1,  0,  0,  0,  0,  0,  0,  0,  0 ]';
	v(:,5)  = [  1, -1,  0,  0,  0,  0,  0,  0,  0,  1,  0,  0,  0,  0,  0,  0,  1,  0,  0 ]';
	v(:,6)  = [  0,  0,  0,  0,  0,  0,  0,  0,  0,  1,  0,  0,  0,  0,  0,  0,  0,  0,  0 ]';
	v(:,7)  = [  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0, -1,  0,  0,  0,  0,  0,  0 ]';
	v(:,8)  = [  0,  0,  0,  0,  0,  0,  0,  0,  0, -1,  0,  0,  0,  0,  0,  0,  0,  0,  0 ]';
	v(:,9) =  [  0,  0,  0, -1,  1,  0,  0,  0,  0, -1,  0,  0,  0,  0,  0,  0,  0,  0,  0 ]';
	v(:,10) = [  0,  0,  0,  1, -1,  0,  0,  0,  0,  1,  0,  0,  0,  0,  0,  0,  0,  0,  0 ]';
	v(:,11) = [  0,  0,  0, -1,  0,  1,  0,  0,  0,  0, -1,  0,  0,  0,  0,  0,  0,  0,  0 ]';
	v(:,12) = [  0,  0,  0,  1,  0, -1,  0,  0,  0,  0,  1,  0,  0,  0,  0,  0,  0,  0,  0 ]';
	v(:,13) = [  0,  0,  0,  1, -1,  0,  0,  0,  0,  1,  0,  0,  0,  0,  0,  0,  0,  1,  0 ]';
	v(:,14) = [  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  1,  0,  0,  0,  0,  0,  0,  0,  0 ]';
	v(:,15) = [  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0, -1,  0,  0,  0,  0,  0 ]';
	v(:,16) = [  0,  0,  0,  0,  0,  0,  0,  0,  0,  0, -1,  0,  0,  0,  0,  0,  0,  0,  0 ]';
	v(:,17) = [  0,  0,  0,  0,  0,  0, -1,  1,  0, -1,  0,  0,  0,  0,  0,  0,  0,  0,  0 ]';
	v(:,18) = [  0,  0,  0,  0,  0,  0,  1, -1,  0,  1,  0,  0,  0,  0,  0,  0,  0,  0,  0 ]';
	v(:,19) = [  0,  0,  0,  0,  0,  0, -1,  0,  1,  0, -1,  0,  0,  0,  0,  0,  0,  0,  0 ]';
	v(:,20) = [  0,  0,  0,  0,  0,  0,  1,  0, -1,  0,  1,  0,  0,  0,  0,  0,  0,  0,  0 ]';
	v(:,21) = [  0,  0,  0,  0,  0,  0,  1, -1,  0,  1,  0,  0,  0,  0,  0,  0,  0,  0,  1 ]';
	v(:,22) = [  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  1,  0,  0,  0,  0,  0,  0,  0 ]';
	v(:,23) = [  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0, -1,  0,  0,  0,  0 ]';
	v(:,24) = [  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0, -1,  0,  0,  0,  0,  0,  0,  0 ]';
	v(:,25) = [  0,  0,  0,  0,  0,  0,  0,  0,  0,  1,  0,  0,  0,  0,  0,  0,  0,  0,  0 ]';
	v(:,26) = [  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  1,  0,  0,  0, -1,  0,  0 ]';
	v(:,27) = [  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  1,  0,  0,  0, -1,  0 ]';
	v(:,28) = [  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  1,  0,  0,  0, -1 ]';
	v(:,29) = [  1,  0, -1,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0 ]';
	v(:,31) = [  0,  0,  0,  1,  0, -1,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0 ]';
	v(:,32) = [  0,  0,  0,  0,  0,  0,  1,  0, -1,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0 ]';


%% chemical rates of reactions above and other system parameters values used during simulation
% What about value units?

%		[  	k01	k10	k19	]
%		[  	k02	k11	k20	]
%		[ 	k03	k12	k21	]
%	 	[ 	k04	k13	k22	]
%	k =	[ 	k05	k14	k23	]
%		[ 	k06	k15	k24	]
%		[ 	k07	k16	k25	]
%		[ 	k08	k17	k26	]
%
%
% Known parameter from literature:
%
%	RNAp T7
%	k01 = 50 1/(uM*s)
%	k02 = 0.2 1/s
%
%
% Chemical rate of biochemical process where the pCMV region
% begins transcription of RNAPT7 is defined outside the k-matrix
%
%	k_cmv

	k = zeros(10,3);

	k(:,1) = [  50,  0.2, 0.031, 0.0002,   200/2600,   0.1,  0.005776226, 0.05,   1000, 0.0005 ]';
	k(:,2) = [  50,  0.2, 0.031, 0.0002,   200/1700,   0.1,  0.005776226, 0.0005, 1000, 0.0005 ]';
	k(:,3) = [  50,  0.2, 0.031, 0.0002,   200/700,    0.1,  0.005776226, 0.0051, 1000, 0.0005 ]';

	k_cmv = 0.01;

	x = [ 1, 0, 0, 500, 0, 0, 2000, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0 ];


%% vector of species concentration/number of molecule
%  initial sistem species concentration (in nM)


	

%% initial definitions (prepairing structures and data for simulation)

%	Time belongs to the sistem response, calculated as: N*tau
%	example 15000*0.2s = 3000s of simulation time
	tau = 0.5;
	N = 1000000;

	change = zeros( 1,length(x) );
	concentration_gfp    = zeros(1,N);
	concentration_nictal = zeros(1,N);
	concentration_rnapt7 = zeros(1,N);

% 	gfp_r = zeros(1,N);
% 	rna_r = zeros(1,N);
% 	nic_r = zeros(1,N);

%% tau-leap simulation

    % measuring processing time
	% average one step of for cycle is 2.88 ms long  -> processing time is then N*2.88ms
	tic();

	for i = 1 : N

		concentration_gfp(i)    = x(12);
		concentration_nictal(i) = x(11);
		concentration_rnapt7(i) = x(10);

% 		rna_r(i) = x(3);
% 		nic_r(i) = x(6);
% 		gfp_r(i) = x(9);

		% evaluation of probabilities
		a = ajx( R, x, k, k_cmv );

		% calculating sistem changing
		for j = 1 : R
			t = v(:,j)' * poissrnd( a(j) * tau );
			% we not allow negative concentration inside the
			% sistem during processing
			if ( min(change + t + x) >= 0 )
			% tau - leap step
				change = change + t;
			end
		end

		% changing species concentrations
		x = x + change;
		change = zeros(1,length(x));

	end

	toc();

%% Plotting results
	figure(30);
	clf
	hold on;
	date = datestr(now);
	title( date );
	%text(100,max([max(concentration_gfp),max(concentration_nictal),max(concentration_rnapt7)])-0.5, num2str(k_cmv) );
	xlabel('Time [s]');
	ylabel('Protein concentration [nM]');
  	plot( linspace(0, floor(N*tau), N), concentration_gfp,    'g' );
 	plot( linspace(0, floor(N*tau), N), concentration_rnapt7, 'r' );
 	plot( linspace(0, floor(N*tau), N), concentration_nictal, 'b' );
% 	plot( linspace(0, floor(N*tau), N), rna_r, 'black' );
%  	plot( linspace(0, floor(N*tau), N), nic_r, 'y' );
%  	plot( linspace(0, floor(N*tau), N), gfp_r, 'm' );
	legend('gfp','rnapt7','nictal');
	hold off;

end


%% evaluation of a_j(x) in time dt = tau
function a = ajx( R, x, k, k_cmv )

	a = zeros(1,R);

	a(1)  = k(1,1)*x(1)*x(10);
	a(2)  = k(2,1)*x(2);
	a(3)  = k(3,1)*x(1)*x(11);
	a(4)  = k(4,1)*x(3);
	a(5)  = k(5,1)*x(2);
	a(6)  = k(6,1)*x(13);
	a(7)  = k(7,1)*x(13);
	a(8)  = k(8,1)*x(10);
	a(9)  = k(1,2)*x(4)*x(10);
	a(10) = k(2,2)*x(5);
	a(11) = k(3,2)*x(4)*x(11);
	a(12) = k(4,2)*x(6);
	a(13) = k(5,2)*x(5);
	a(14) = k(6,2)*x(14);
	a(15) = k(7,2)*x(14);
	a(16) = k(8,2)*x(11);
	a(17) = k(1,3)*x(7)*x(10);
	a(18) = k(2,3)*x(8);
	a(19) = k(3,3)*x(7)*x(11);
	a(20) = k(4,3)*x(9);
	a(21) = k(5,3)*x(8);
	a(22) = k(6,3)*x(15);
	a(23) = k(7,3)*x(15);
	a(24) = k(8,3)*x(12);
	a(25) = k_cmv * x(16);
	a(26) = k(9,1)*x(17);
	a(27) = k(9,2)*x(18);
	a(28) = k(9,3)*x(19);
	a(29) = k(10,1)*x(3);
	a(30) = k(10,2)*x(6);
	a(31) = k(10,3)*x(9);

	% at the end we need only to normalize probabilties
	a = a/max(a);
end






